package model

import (
	"fmt"
	"github.com/fonchain_enterprise/fonchain-approval/api/approval"
	md52 "github.com/fonchain_enterprise/fonchain-approval/pkg/common/md5"
	"gorm.io/plugin/soft_delete"
	"math/rand"
	"strconv"
	"time"
)

// ApprovalType 审批类型
type ApprovalType struct {
	ID                uint64                `gorm:"primaryKey;column:id" json:"id"`                                         // ID
	DeletedAt         soft_delete.DeletedAt `gorm:"column:deleted_at" json:"deletedAt"`                                     // 删除时间
	CreatedAt         time.Time             `gorm:"column:created_at" json:"createdAt"`                                     // 创建时间
	UpdatedAt         time.Time             `gorm:"column:updated_at" json:"updatedAt"`                                     // 更新时间
	KeyWord           string                `gorm:"column:key_word" json:"keyWord"`                                         // 回复内容
	IsNormal          uint64                `gorm:"type:tinyint;column:is_normal;default:1;comment:是否是自定义" json:"isNormal"` // 回复内容
	GroupId           uint64                `gorm:"type:int(11);column:group_id;default:0;comment:类型的分组" json:"groupId"`    // 回复内容
	KeyJson           KeyInfos              `gorm:"type:json;column:key_json;default:null;comment:自定义字段" json:"keyJson"`
	Domain            string                `gorm:"column:domain" json:"domain"`                 //
	Title             string                `gorm:"column:title" json:"title"`                   // 标题
	Remark            string                `gorm:"column:remark" json:"remark"`                 // 备注
	ApprovalTypeGroup *ApprovalTypeGroup    `gorm:"foreignKey:GroupId" json:"ApprovalTypeGroup"` // 绑定的分租
}

// TableName get sql table name.获取数据库表名
func (m *ApprovalType) TableName() string {
	return "approval_type"
}

var TypeMap = map[string]string{
	"content":               "",
	"work":                  "",
	"exhibition":            "",
	"bundle":                "",
	"show":                  "",
	TypeBundlePayPrice:      "",
	TypeArtExhibitionReward: "",

	"leave":            "", // 事假
	"sick":             "", // 病假
	"annualLeave":      "", // 年假
	"dayOff":           "", // 调休
	"maritalLeave":     "", // 婚假
	"matingCheckLeave": "", // 孕检假
	"maternityLeave":   "", // 产假
	"paternityLeave":   "", // 陪产假
	"parentalLeave":    "", // 育儿假
	"nursingLeave":     "", // 独生子女护理假
	"funeralLeave":     "", // 丧假
	"makeUp":           "", // 补卡
	"overTime":         "", // 加班
	"outWork":          "", // 外勤
	"turnover":         "", // 离职

	"financialFormApply": "", // 财务表单审批
}

var keyWordMap = map[string]string{
	"普通审批":     "content",
	"画作申请审批":   "work",
	"画展包审批":    "exhibition",
	"画展包数量审批":  "bundle",
	"画展审批":     "show",
	"画展包付款单申请": TypeBundlePayPrice,
	"画作润格审批":   TypeArtExhibitionReward,
	"事假":       "leave",              // 事假
	"病假":       "sick",               // 病假
	"年假":       "annualLeave",        // 年假
	"调休":       "dayOff",             // 调休
	"婚假":       "maritalLeave",       // 婚假
	"孕检假":      "matingCheckLeave",   // 孕检假
	"产假":       "maternityLeave",     // 产假
	"陪产假":      "paternityLeave",     // 陪产假
	"育儿假":      "parentalLeave",      // 育儿假
	"独生子女护理假":  "nursingLeave",       // 独生子女护理假
	"丧假":       "funeralLeave",       // 丧假
	"补卡":       "makeUp",             // 补卡
	"加班":       "overTime",           // 加班
	"外勤":       "outWork",            // 外勤
	"离职":       "turnover",           // 离职
	"财务表单审批":   "financialFormApply", // 财务表单审批
}

func DataExist(domain, keyWork, title string) bool {
	var count int64

	DB.Model(&ApprovalType{}).Where(&ApprovalType{Domain: domain, Title: title}).Count(&count)
	if count != 0 {
		return true
	}

	DB.Model(&ApprovalType{}).Where(&ApprovalType{Domain: domain, KeyWord: keyWork}).Count(&count)
	if count != 0 {
		return true
	}

	return false

}

func ToKeyInfos(infos []*approval.KeyInfo) KeyInfos {
	var info KeyInfos

	for index, t := range infos {

		temp := KeyInfo{
			ApprovalKey:   t.ApprovalKey,
			ApprovalName:  t.ApprovalName,
			ApprovalType:  t.ApprovalType,
			ApprovalValue: t.ApprovalValue,
			ApprovalMark:  t.ApprovalMark,
		}

		if temp.ApprovalKey == "" {
			temp.ApprovalKey = "key_" + fmt.Sprintf("%d", index)
		}

		info = append(info, temp)
	}

	return info
}

func KeyWordFromTitle(title string) string {
	keyWord := md52.Md5Part(title+strconv.Itoa(rand.Intn(100)), 8, 16)

	if IsSystemKeyWork(title) == true {
		keyWord = keyWordMap[title]
	}

	return keyWord

}

func IsSystemKeyWork(title string) bool {
	_, isOk := keyWordMap[title]
	return isOk
}

func IsSystemType(title string) bool {
	_, isOk := TypeMap[title]
	return isOk
}
